#include "planets.h"
#include "reperes.h"
#include "terre.h"
#include "../mathutil/sp3.h"
#include <math.h>


// CLASSE STATIQUE. Pas besoinde créer d'instance.

/** Si true, calcul dans le référentiel fixe J2000, sinon dans le reférentiel de l'équinoxe moyen de la date */
int Planets::refJ2000 = 0;

/** Si true, methode spherique, sinon, methode cartesienne */
int Planets::methodeSpherique = 1;

// Pour pifométrer les magnitudes
#define moyMercure 1.7
//#define ampMercure 3.8
#define moyVenus -4.2
//#define ampVenus 0.4
#define moyMars -0.55
//#define ampMars 2.35
#define moyJupiter -2.2
//#define ampJupiter 0.6
#define moySaturne 0.48
//#define ampSaturne 0.72

const static double moys[6] = {moyMercure, moyVenus, 0, moyMars, moyJupiter, moySaturne};
const static double amps[6] = {3.8, 0.4, 0, 2.35, 0.6, 0.72};
/** Déplacement angulaire par jour */
#define dp (2.*M_PI)
const static double errAng[8] = {dp/87.5, dp/224.7, dp/365.25, dp/687.,
                                 dp/4331., dp/10747., dp/30589., dp/59802.};

/** Magnitudes apparentes pifométrées */
double Planets::magpla[8] = {moyMercure, moyVenus, 0, moyMars, moyJupiter, moySaturne, 5.3, 8};

/** Positions des planètes par rapport au Soleil dans Re, et lamba, beta, rho */
double Planets::v8x6_OP_e[8][6];

/** Positions des planètes par rapport à la Terre dans Rc */
double Planets::v8x3_TP_c[8][3];

/** Nom des 8 planètes */
QString Planets::noms[8] = {"Mercure", "Vénus", "Terre", "Mars", "Jupiter", "Saturne", "Uranus", "Neptune"};


/** Configuration repere de calcul et mode de calcul */
void Planets::config(bool ref, bool sphere)
{
    refJ2000 = ref ? 1 : 0;
    methodeSpherique = sphere ? 1 : 0;
}

// Instant improbable
#define tW -1.e+20
double Planets::tcou = tW;
double Planets::tpos[8] = {tW, tW, tW, tW, tW, tW, tW, tW};

/** Positions à l'instant jt compté en secondes après le 1er janvier 2000 a midi a Greenwich (J2000.0). */
void Planets::etat(double jours, double precision)
{
    if (tcou == jours) return;
    tcou = jours;
    int i, k;
    double T = jours/36525.0; // en siècles juliens depuis J2000.
    /* Positions des planètes relativement au Soleil, dans Re */
    for (i = 0; i < 8; i++)
    {
        if (fabs(jours - tpos[i])*errAng[i] < precision) continue;
        tpos[i] = jours;
        pospla(v8x6_OP_e[i], T, i, refJ2000, methodeSpherique);
    }
    /* Positions des planètes relativement a la Terre, dans Re */
    for (i = 0; i < 8; i++)
    {
        if(i==2) // Cas de la Terre, on y met le Soleil.
        {
            for(k = 0; k < 3; k++) v8x3_TP_c[i][k] =  - v8x6_OP_e[2][k];
        }
        else for(k = 0; k < 3; k++) v8x3_TP_c[i][k] =  v8x6_OP_e[i][k] -  v8x6_OP_e[2][k];
    }
    /* Projection dans Rc et conversion en mètres */
    for (i = 0; i < 8; i++)
    {
        m3v3(v8x3_TP_c[i], Reperes::m3_CE, v8x3_TP_c[i]);
        scav3(v8x3_TP_c[i],Terre::UA, v8x3_TP_c[i]);
    }
    /* Magnitude pifométrées des 6 premières planètes */
    /* Pour les planètes inférieures : maximale qd proche de la Terre et minimale qd de l'autre coté du Soleil
     * Pour les planètes supérieures : maximales en quadrature et minimales à l'alignement */
    for (i = 0; i < 6; i++)
    {
        if (i == 2) continue;
        double cosalf = 0.; // produit scalaire OP.OT
        for (k=0; k <3; k++) cosalf += v8x6_OP_e[i][k] *  v8x6_OP_e[2][k];
        cosalf /= v8x6_OP_e[i][5] *  v8x6_OP_e[2][5];
        if (i < 2)  magpla[i] = moys[i] - amps[i]*cosalf; // planètes inférieures
        else {magpla[i] = moys[i] + amps[i]*(2*cosalf*cosalf -1.); } // Planètes supérieures
    }
    return;
}

void Planets::test()
{
	return;
}
