#include "timedialog.h"
#include "ui_timedialog.h"
#include "../utils/cjulien/julien.h"
#include "cinespace.h"

TimeDialog::TimeDialog(CineSpace *s, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::TimeDialog)
{
    ui->setupUi(this);
    notinitok = true;
    stars = s;

    ui->comboBox_scenar->addItem(tr("Sans modification"));
    ui->comboBox_scenar->addItem(tr("Synchro avec levers"));
    ui->comboBox_scenar->addItem(tr("Synchro avec couchers"));

    labelHelpOrd = new QLabel(
            tr(
                    "<h4>Ordonnancement :</h4>"
                    "<ul>La simulation se déroule entre la date début et la date fin.</ul>"
                    "<h4>Accélération : </h4>"
                    "<ul>Coef. multiplicateur du temps pour accélérer la simulation : c'est la durée de<br/>"
                    "temps simulée pour 1 seconde de temps usager.</ul>"
                    "<h4>Tranches d'observation :</h4>"
                    "<ul>La simulation peut être découpée en tranches de durée 'période' dans lesquelles<br/>"
                    "seul un premier intervalle de temps de durée 'creneau' est simulé, puis on saute<br/>"
                    "instantanément à la tranche suivante. Pas de sauts si creneau >= tranche.<br/>"
                    "Attention :"
                    "<li>Période : est exprimé en jours, par exemple 1 (jour).</li>"
                    "<li>creneau : est exprimé en secondes, par exemple 86400 (secondes).</li>"
                    "<li>Décalage temporel : décalage de l'instant de début de simulation dans chaque<br/>"
                    "tranche. Utile dans le cas des scenarios 'Levers ou 'Couchers' pour décaler le début<br/>"
                    "de l'observation par rapports aux instants de lever ou coucher du Soleil.</li></ul>"
                    "<h4>Scenario :</h4>"
                    "<ul><li>Aucun : Les instants de début de chaque tranche sont des multiples exacts de période.</li>"
                    "<li>Levers : Modification de l'instant de début de tranche pour qu'elle débute au lever<br/>"
                    "du Soleil le plus proche.</li>"
                    "<li>Couchers : Modification de l'instant de début de tranche pour qu'elle débute au coucher<br/>"
                    "du Soleil le plus proche.</li></ul>"
                    "<h4>Synchro</h4>"
                    "<ul>Envoi des paramètres à la simulation.</ul>"
                    ), 0);
    labelHelpOrd->setWindowTitle("Aide ordonnancement");

    init();
    notinitok = false;

}

TimeDialog::~TimeDialog()
{
    labelHelpOrd->close();
    delete ui;
}

void TimeDialog::closeEvent(QCloseEvent *event)
{
    labelHelpOrd->close();
}

void TimeDialog::init()
{
    int date[6];

    ui->comboBox_scenar->setCurrentIndex((int) stars->scenar);

    jjDeb = stars->tdeb/86400. + Date_J2000_j;
    ui->doubleSpinBox_jjDeb->setValue(jjDeb);

    jul2tdat(date, jjDeb);
    aDeb = date[0]; moDeb = date[1]; jDeb = date[2]; hDeb = date[3]; mnDeb = date[4]; sDeb = date[5];
    update_dateDeb();

    jjFin = stars->tfin/86400. + Date_J2000_j;
    ui->doubleSpinBox_jjFin->setValue(jjFin);

    jul2tdat(date, jjFin);
    aFin = date[0]; moFin = date[1]; jFin = date[2]; hFin = date[3]; mnFin = date[4]; sFin = date[5];
    update_dateFin();

    accel = stars->accel;
    creneau = stars->creneau;
    decalage = stars->decalage;
    periode = stars->periode;
    ui->doubleSpinBox_accel->setValue(accel);
    ui->doubleSpinBox_jcreneau->setValue(creneau);
    ui->doubleSpinBox_decalTime->setValue(decalage);
    ui->doubleSpinBox_jperiode->setValue(periode/86400.);
}

void TimeDialog::setAccel(double acc)
{
    accel = acc;
    ui->doubleSpinBox_accel->setValue(accel);
}

void TimeDialog::setCreneau(double cre)
{
    creneau = cre;
    ui->doubleSpinBox_jcreneau->setValue(creneau);
}

void TimeDialog::updateCourant(double time)
{
    int date[6];

    double jul = time/86400. + Date_J2000_j;
    ui->l_julien->setText(QString::number(jul, 'f', 4));
    const char *s = jul2tdat(date, jul);
    ui->l_strJour->setText(s);
    ui->l_jour->setText(QString::number(date[2]));
    ui->l_mois->setText(QString::number(date[1]));
    ui->l_annee->setText(QString::number(date[0]));
    ui->l_heure->setText(QString::number(date[3]));
    ui->l_minute->setText(QString::number(date[4]));
    ui->l_seconde->setText(QString::number(date[5]));
}

void TimeDialog::update_dateDeb()
{
    ui->spinBox_jourDeb->setValue(jDeb);
    ui->spinBox_moisDeb->setValue(moDeb);
    ui->spinBox_anneeDeb->setValue(aDeb);
    ui->spinBox_hDeb->setValue(hDeb);
    ui->spinBox_mDeb->setValue(mnDeb);
    ui->spinBox_sDeb->setValue(sDeb);
}

void TimeDialog::update_dateFin()
{
    ui->spinBox_jourFin->setValue(jFin);
    ui->spinBox_moisFin->setValue(moFin);
    ui->spinBox_anneeFin->setValue(aFin);
    ui->spinBox_hFin->setValue(hFin);
    ui->spinBox_mFin->setValue(mnFin);
    ui->spinBox_sFin->setValue(sFin);
}


void TimeDialog::update_jjDeb()
{
    double dj1 = jDeb + (hDeb+(mnDeb+sDeb/60.)/60.)/24.;
    jjDeb = dat2jul(dj1, moDeb, aDeb);
    ui->doubleSpinBox_jjDeb->setValue(jjDeb);
    ui->label_jourDeb->setText(jul2jour(jjDeb));
}

void TimeDialog::update_jjFin()
{
    double dj2 = jFin + (hFin+(mnFin+sFin/60.)/60.)/24.;
    jjFin = dat2jul(dj2, moFin, aFin);
    ui->doubleSpinBox_jjFin->setValue(jjFin);
    ui->label_jourFin->setText(jul2jour(jjFin));
}

void TimeDialog::on_spinBox_jourDeb_editingFinished()
{
    jDeb = ui->spinBox_jourDeb->value();
    update_jjDeb();
}

void TimeDialog::on_spinBox_moisDeb_editingFinished()
{
    moDeb = ui->spinBox_moisDeb->value();
    update_jjDeb();
}

void TimeDialog::on_spinBox_anneeDeb_editingFinished()
{
    aDeb = ui->spinBox_anneeDeb->value();
    update_jjDeb();
}

void TimeDialog::on_spinBox_hDeb_editingFinished()
{
    hDeb = ui->spinBox_hDeb->value();
    update_jjDeb();
}

void TimeDialog::on_spinBox_mDeb_editingFinished()
{
    mnDeb = ui->spinBox_mDeb->value();
    update_jjDeb();
}

void TimeDialog::on_spinBox_sDeb_editingFinished()
{
    sDeb = ui->spinBox_sDeb->value();
    update_jjDeb();
}

void TimeDialog::on_doubleSpinBox_jjDeb_editingFinished()
{
    int date[6];
    jjDeb = ui->doubleSpinBox_jjDeb->value();
    ui->label_jourDeb->setText(jul2tdat(date, jjDeb));
    aDeb = date[0]; moDeb = date[1]; jDeb = date[2]; hDeb = date[3]; mnDeb = date[4]; sDeb = date[5];
    update_dateDeb();
}

void TimeDialog::on_spinBox_jourFin_editingFinished()
{
    jFin = ui->spinBox_jourFin->value();
    update_jjFin();
}

void TimeDialog::on_spinBox_moisFin_editingFinished()
{
    moFin = ui->spinBox_moisFin->value();
    update_jjFin();
}

void TimeDialog::on_spinBox_anneeFin_editingFinished()
{
    aFin = ui->spinBox_anneeFin->value();
    update_jjFin();
}

void TimeDialog::on_spinBox_hFin_editingFinished()
{
    hFin = ui->spinBox_hFin->value();
    update_jjFin();
}

void TimeDialog::on_spinBox_mFin_editingFinished()
{
    mnFin = ui->spinBox_mFin->value();
    update_jjFin();
}

void TimeDialog::on_spinBox_sFin_editingFinished()
{
    sFin = ui->spinBox_sFin->value();
    update_jjFin();
}

void TimeDialog::on_doubleSpinBox_jjFin_editingFinished()
{
    int date[6];
    jjFin = ui->doubleSpinBox_jjFin->value();
    ui->label_jourFin->setText(jul2tdat(date, jjFin));
    aFin = date[0]; moFin = date[1]; jFin = date[2]; hFin = date[3]; mnFin = date[4]; sFin = date[5];
    update_dateFin();
}



void TimeDialog::on_doubleSpinBox_accel_valueChanged(double v)
{
    accel = v;
}

void TimeDialog::on_doubleSpinBox_jcreneau_valueChanged(double v)
{
    creneau = v;
}

void TimeDialog::on_doubleSpinBox_decalTime_valueChanged(double v)
{
    decalage = v;
}

void TimeDialog::on_doubleSpinBox_jperiode_valueChanged(double v)
{
    periode = v*86400.;
}

void TimeDialog::on_pushButton_Synchro_clicked()
{
    stars->tfin = (jjFin - Date_J2000_j)*86400.;
    stars->tdeb = (jjDeb - Date_J2000_j)*86400.;
    stars->accel = accel;
    stars->creneau = creneau;
    stars->decalage = decalage;
    stars->periode = periode;
    emit doSynchro();
}

void TimeDialog::on_comboBox_scenar_currentIndexChanged(int index)
{
    if(notinitok) return;
    stars->scenar = (Scenario) index;
    emit doSynchro();
}

void TimeDialog::on_pushButton_Aide_clicked()
{
    labelHelpOrd->show();
}
