#ifndef SP3_H
#define SP3_H
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/**********************************************************************/
/*                         UTILITAIRES MATRICIELS                     */
/**********************************************************************/

/***************************************************************************/
/************************  ESPACE DIMENSION 3  *****************************/
/***************************************************************************/
/* toString d'un vecteur de R3 */
char *sprintv3(char *sbuf, double *a, int n, int d) ;
/* Impression d'un vecteur de R3 */
void printv3(char *s, double *a) ;
/* Idem en spécifiant fichier de sortie et format pour les nombres 
   genre : "%11.3lf" ou "%11.3lg" */
void fprintv3(char *s, double *a, FILE *fich, char *format);

/* Impression d'une matrice de R3 */
void printm3(char *s, double *a);

double *setv3 (double* u, double s);

double *iniv3(double *v, double x, double y, double z);

double *cpyv3(double* u, double* v);

double *negv3(double* u, double* v);

double *absv3(double* u, double* v);

/* Composante maximale d'un vecteur de R3 : s = max(Ui) */
double xmaxv3(double* u);

/* Composante max. en val. abs. d'un vecteur de R3 : s = max(|Ui|) */
double xamaxv3(double* u);

/* Composante max. en val. abs.d'un vecteur de R3 : s = max(Ui) */
double xamaxv3(double* u);

/* Composante minimale d'un vecteur de R3 : s = min(Ui) */
double xminv3(double* u);

/* Composante min. en val. abs. d'un vecteur de R3 : s = min(|Ui|) */
double xaminv3(double* u);

/* Index de la composante max d'un vecteur de R3 */
int imaxv3(double* u);

/* Index de la composante max en v.abs. d'un vecteur de R3 */
int iamaxv3(double* u);

/* Index de la composante min d'un vecteur de R3 */
int iminv3(double* u);

/* Index de la composante min en v.abs. d'un vecteur de R3 */
int iaminv3(double* u);

/* Somme des composantes d'un vecteurs de R3 : s = Sigma Ui */
double xsumv3(double* u);

double *scav3(double* u, double s, double* v);

double *addv3(double* u, double* v, double* w);

double *subv3(double* u, double* v, double* w);

/* Multiplication terme a terme de vecteurs de R3 : U = V x W */
double *mulv3(double* u, double* v, double* w);

/* Division terme a terme de vecteurs de R3 : U = V / W */
double *divv3(double* u, double* v, double* w);

/* Cumulation de vecteurs de R3 : U = V + aW */
double *cumv3(double* u, double* v,double a, double* w);

/* Produit scalaire de vecteurs de R3 : s = <A . B> */
double dotv3(double* a, double* b) ;

/* Produit tensoriel de vecteurs de R3 : M = V1 * V2' */
double *tensv3 (double *mato, double *v1, double *v2);

/* Produit vectoriel de vecteurs de R3 : A = B ^ C */
double *crossv3(double* a, double* b, double* c); /* A != B ou C */

/* Carré d'un vecteurs de R3 : s = A'*A */
double carrev3(double* a);

/* Longueur d'un vecteurs de R3 : s = sqrt(A'*A) */
double longv3(double* a);

/* Normalisation d'un vecteur de R3 : A = B / ||B|| */
double *unitv3(double* a, double* b);

/* Normalisation d'un vecteur de R3 : A = B / ||B|| */
double lunitv3(double* a, double* b);

double *setm3(double* u, double s);

double *cpym3(double* u, double* v);

double *negm3(double* u, double* v);

double *addm3(double* u, double* v, double *w);

double *subm3(double* u, double* v, double *w);

double *scam3(double* u, double s, double* v);

double *cumm3(double* u, double* v, double s, double *w);

double *eyem3(double *a);

/* Génération d'une matrice diagonale à partir d'un vecteur de R3 */
double *diag2m3(double *a, double *v);

/* Extraction du vecteur diagonal d'une matrice de R3 */
double *m3todiag(double *v, double *a);

/* Génération matrice antisymétrique à partir d'un vecteur de R3 */
double *v3toasm3(double *a, double *v);

/* Extraction du vecteur antisymétrique d'une matrice de R3 */
double *asm3tov3(double *v, double *a);

/* Transposée d'une matrice de R3 : A = A' */
double *tm3(double *a);

/* Produit matrice x vecteurs de R3 : w = A v */
double *m3v3(double* w, double* a, double* v);

/* Produit matrice transposée x vecteurs de R3 : w = A' v */
double *tm3v3(double* w, double* a, double* v);

/* Produit vecteur transposé x matrice de R3 : a' = b' C */
double *tv3m3( double* a,double* b, double* c);

/* Produit matrice x matrice de R3 : A = B C */
double *m3m3(double* a,double* b, double* c);

/* Produit matrice transposée x matrice de R3 : A = B'C */
double *tm3m3( double* a,double* b, double* c);

/* Produit matrice x matrice transposée de R3 : A = B C' */
double *m3tm3(double* a, double* b, double* c);

/* Trace d'une matrice de R3 : s = Sigma de Aii */
double tracem3(double* a);

/* Déterminant d'une matrice de R3 : s = |A| */
double detm3(double* a);

/* Inverse d'une matrice de R3 : inv(A)
   ATTENTION : EXTREMEMENT DANGEREUX SI DETERMINANT NUL
   FAIRE PRECEDER DU CALCUL DU DETERMINANT
*/
double *invm3(double* a);

/* Déterminant et Inverse d'une matrice de R3 : s = |A|, inv(A) */
double dinvm3(double* a);

/* Impression d'une matrice de R3 */
void printm3(char *s, double *a);

/*  Matrice d'orientation d'un repère de visée */
void asiTom3(double *mat, double azi, double sit);

/* Matrice d'attitue d'un repère local sur Terre (X vers le Nord) */
void lolaTom3(double *mat, double lon, double lat);

/* Matrice d'attitue d'un repère local sur Terre (X vers le Sud)*/
void lolaAstroTom3(double *mat, double lon, double lat);

/* Conversion matrice 3x3 en matrice homogène 4x4 type openGL */
double *m3ToGLmat44(double *m44,double *mat);

/* Conversion matrice 3x3 transposée en matrice homogène 4x4 type openGL */
double *tm3ToGLmat44(double *m44,double *mat);
#ifdef __cplusplus
}
#endif

#endif
