#ifndef ANGLES_H
#define ANGLES_H

#ifdef __cplusplus
extern "C" {
#endif


// Conversion degres decimaux en HH MM SS.S et string
// Usage : str = deg2hms(buf, ang, &hh, &mhm, &ss);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en degre
// Sortie : str = HHhMMmSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *deg2hms(char *buffer, double ang, int *h, int *m, double *s);

// Conversion radians en HH MM SS.S et string
// Usage str = rad2hms(buf, ang, &h, &m, &s);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en radian
// Sortie : str = string HHhMMhSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999... 
char *rad2hms(char *buffer, double ang, int *h, int *m, double *s);

// Conversion degres, minutes secondes en degres decimaux
// Usage : ang = dms2deg(d, m, s);
double dms2deg(int d, int m, double s);

// Conversion heures, minutes secondes en heures decimales
// Usage : hours = hms2hours(h, m, s);
double hms2hours(int h, int m, double s);

// Conversion degres, minutes secondes en radians
// Usage : ang = dms2rad(d, m, s);
double dms2rad(int d, int m, double s);

// Conversion heures, minutes secondes en degres decimaux
// Usage : ang = hms2deg(h, m, s)
double hms2deg(int h, int m, double s);

// Conversion heures, minutes secondes en jour décimal
// Usage : jours = hms2jours(h, m, s)
double hms2jours(int h, int m, double s);

// Conversion heures, minutes secondes en radians
// Usage : ang = hms2rad(h, m, s)
double hms2rad(int h, int m, double s);

// Conversion jour décimal en heure minute seconde
// Usage : str = jours2hms(buffer, jours, &h, &m, &s);
// Entree : buffer : buffer pour le string resultat
//			ang = angle en radians
// Sortie : str = JJJjHHhMMmSS.Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *jours2hms(char *buffer, double jours, int *j, int *h, int *m, double *s);

// Conversion radian vers Degrees MM SS.S
//  Usage str = rad2dms(buffer, ang, &d, &m, &s)
// Entree : buffer : buffer pour le string resultat
//			ang = angle en radians
// Sortie : str = DDD°MM'SS".Ss
//          hh = heures 0 à 23h
//          mm = minutes O à 59
//          ss = secondes décimales 0 à 59.9999...
char *rad2dms(char *buffer, double ang, int *d, int *m, double *s);

#ifdef __cplusplus
}
#endif

#endif //#ifndef ANGLES_H
